@@ -12,7 +12,7 @@ const Header = require('../lib/header.js')
 const zlib = require('zlib')
 const miniz = require('minizlib')
 const mutateFS = require('mutate-fs')
-const MiniPass = require('minipass')
+const { Minipass } = require('minipass')
 process.env.USER = 'isaacs'
 const EE = require('events').EventEmitter
 const rimraf = require('rimraf')
@@ -375,6 +375,19 @@ t.test('if gzip is truthy, make it an object', t => {
   t.end()
 })
 
+t.test('if brotli is truthy, make it an object', t => {
+  const opt = { brotli: true }
+  new Pack(opt)
+  t.type(opt.brotli, 'object')
+  t.end()
+})
+
+t.test('throws if both gzip and brotli are truthy', t => {
+  const opt = { gzip: true, brotli: true }
+  t.throws(_ => new Pack(opt), new TypeError('gzip and brotli are mutually exclusive'))
+  t.end()
+})
+
 t.test('gzip, also a very deep path', t => {
   const out = []
 
@@ -454,6 +467,84 @@ t.test('gzip, also a very deep path', t => {
     })
 })
 
+t.test('brotli, also a very deep path', t => {
+  const out = []
+
+  new Pack({
+    cwd: files,
+    brotli: { flush: 1 },
+  })
+    .add('dir')
+    .add('long-path')
+    .on('data', c => out.push(c))
+    .end()
+    .on('end', _ => {
+      const zipped = Buffer.concat(out)
+      const data = zlib.brotliDecompressSync(zipped)
+      const entries = []
+      for (var i = 0; i < data.length; i += 512) {
+        const slice = data.slice(i, i + 512)
+        const h = new Header(slice)
+        if (h.nullBlock) {
+          entries.push('null block')
+        } else if (h.cksumValid) {
+          entries.push([h.type, h.path])
+        } else if (entries[entries.length - 1][0] === 'File') {
+          entries[entries.length - 1].push(slice.toString().replace(/\0.*$/, ''))
+        }
+      }
+
+      const expect = [
+        ['Directory', 'dir/'],
+        ['Directory', 'long-path/'],
+        ['File', 'dir/x'],
+        ['Directory', 'long-path/r/'],
+        ['Directory', 'long-path/r/e/'],
+        ['Directory', 'long-path/r/e/a/'],
+        ['Directory', 'long-path/r/e/a/l/'],
+        ['Directory', 'long-path/r/e/a/l/l/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/'],
+        ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/'],
+        ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/a.txt', 'short\n'],
+        ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc', '1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'],
+        ['ExtendedHeader', 'PaxHeader/ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'],
+        ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc', '2222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222'],
+        ['ExtendedHeader', 'PaxHeader/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxccccccccccccccccccccccccccccccccccccccc'],
+        ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxccccccccccccccccccccccccccccccccccccccccccccccccc', 'cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'],
+        ['ExtendedHeader', 'PaxHeader/Ω.txt'],
+        ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/Ω.txt', 'Ω'],
+        'null block',
+        'null block',
+      ]
+
+      let ok = true
+      entries.forEach((entry, i) => {
+        ok = ok &&
+                    t.equal(entry[0], expect[i][0]) &&
+                    t.equal(entry[1], expect[i][1]) &&
+                    (!entry[2] || t.equal(entry[2], expect[i][2]))
+      })
+
+      t.end()
+    })
+})
+
 t.test('very deep gzip path, sync', t => {
   const pack = new PackSync({
     cwd: files,
@@ -533,6 +624,84 @@ t.test('very deep gzip path, sync', t => {
   t.end()
 })
 
+t.test('very deep brotli path, sync', t => {
+  const pack = new PackSync({
+    cwd: files,
+    brotli: true,
+  }).add('dir')
+    .add('long-path')
+    .end()
+
+  // these do nothing!
+  pack.pause()
+  pack.resume()
+
+  const zipped = pack.read()
+  t.type(zipped, Buffer)
+  const data = zlib.brotliDecompressSync(zipped)
+  const entries = []
+  for (var i = 0; i < data.length; i += 512) {
+    const slice = data.slice(i, i + 512)
+    const h = new Header(slice)
+    if (h.nullBlock) {
+      entries.push('null block')
+    } else if (h.cksumValid) {
+      entries.push([h.type, h.path])
+    } else if (entries[entries.length - 1][0] === 'File') {
+      entries[entries.length - 1].push(slice.toString().replace(/\0.*$/, ''))
+    }
+  }
+
+  const expect = [
+    ['Directory', 'dir/'],
+    ['File', 'dir/x'],
+    ['Directory', 'long-path/'],
+    ['Directory', 'long-path/r/'],
+    ['Directory', 'long-path/r/e/'],
+    ['Directory', 'long-path/r/e/a/'],
+    ['Directory', 'long-path/r/e/a/l/'],
+    ['Directory', 'long-path/r/e/a/l/l/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/'],
+    ['Directory', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/'],
+    ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/a.txt', 'short\n'],
+    ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc', '1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'],
+    ['ExtendedHeader', 'PaxHeader/ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'],
+    ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc', '2222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222'],
+    ['ExtendedHeader', 'PaxHeader/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxccccccccccccccccccccccccccccccccccccccc'],
+    ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxccccccccccccccccccccccccccccccccccccccccccccccccc', 'cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'],
+    ['ExtendedHeader', 'PaxHeader/Ω.txt'],
+    ['File', 'long-path/r/e/a/l/l/y/-/d/e/e/p/-/f/o/l/d/e/r/-/p/a/t/h/Ω.txt', 'Ω'],
+    'null block',
+    'null block',
+  ]
+
+  let ok = true
+  entries.forEach((entry, i) => {
+    ok = ok &&
+      t.equal(entry[0], expect[i][0]) &&
+    t.equal(entry[1], expect[i][1]) &&
+    (!entry[2] || t.equal(entry[2], expect[i][2]))
+  })
+
+  t.end()
+})
+
 t.test('write after end', t => {
   const p = new Pack()
   p.end()
@@ -563,8 +732,8 @@ t.test('readdir fail', t => {
 
 t.test('pipe into a slow reader', t => {
   const out = []
-  const mp = new MiniPass()
-  const mp2 = new MiniPass()
+  const mp = new Minipass()
+  const mp2 = new Minipass()
   const p = new Pack({ cwd: files }).add('long-path').end()
   p.pause()
   p.pipe(mp).pipe(mp2)
@@ -1135,6 +1304,7 @@ t.test('prefix and hard links', t => {
       cwd: dir + '/in',
       prefix: 'out/x',
       noDirRecurse: true,
+      jobs: 1,
     })
     const out = []
     p.on('data', d => out.push(d))
